@@ -72,7 +72,8 @@ send_and_allow_reentry (DBusConnection * bus, DBusMessage * message)
   dbus_pending_call_set_notify (pending, set_reply, (void *) &closure, NULL);
   closure.loop = g_main_loop_new (NULL, FALSE);
 
-  if (getenv ("AT_SPI_CLIENT"))
+  /* TODO: Remove old AT_SPI_CLIENT name */
+  if (getenv ("AT_SPI_CLIENT") || getenv ("AT_SPI_REENTER_G_MAIN_LOOP"))
     {
       g_main_loop_run  (closure.loop);
     }
@@ -292,6 +293,74 @@ signal_name_to_dbus (const gchar *s)
   return ret;
 }
 
+/*
+ * Converts names of the form "active-descendant-changed" to
+ * "ActiveDescendantChanged"
+ */
+static gchar *
+ensure_proper_format (const char *name)
+{
+  gchar *ret = (gchar *) g_malloc (strlen (name) * 2 + 2);
+  gchar *p = ret;
+  gboolean need_upper = TRUE;
+
+  if (!ret)
+    return NULL;
+  while (*name)
+    {
+      if (need_upper)
+        {
+          *p++ = toupper (*name);
+          need_upper = FALSE;
+        }
+      else if (*name == '-')
+        need_upper = TRUE;
+      else if (*name == ':')
+        {
+          need_upper = TRUE;
+          *p++ = *name;
+        }
+      else
+        *p++ = *name;
+      name++;
+    }
+  *p = '\0';
+  return ret;
+}
+
+static gboolean
+signal_is_needed (const gchar *klass, const gchar *major, const gchar *minor)
+{
+  gchar *data [4];
+  GList *iter;
+  event_data *evdata;
+  gboolean ret = FALSE;
+  GList *list;
+
+  data [0] = ensure_proper_format (klass + 21);
+  data [1] = ensure_proper_format (major);
+  data [2] = ensure_proper_format (minor);
+  data [3] = NULL;
+
+  /* Hack: events such as "object::text-changed::insert:system" as
+     generated by Gecko */
+  data [2][strcspn (data [2], ":")] = '\0';
+  for (list = spi_global_app_data->events; list; list = list->next)
+    {
+      evdata = list->data;
+      if (spi_event_is_subtype (data, evdata->data))
+        {
+          ret = TRUE;
+          break;
+        }
+    }
+
+  g_free (data [2]);
+  g_free (data [1]);
+  g_free (data [0]);
+  return ret;
+}
+
 /*
  * Emits an AT-SPI event.
  * AT-SPI events names are split into three parts:
@@ -326,6 +395,9 @@ emit_event (AtkObject  *obj,
   if (!minor) minor = "";
   if (!type) type = "u";
 
+  if (!signal_is_needed (klass, major, minor))
+    return;
+
   /*
    * This is very annoying, but as '-' isn't a legal signal
    * name in D-Bus (Why not??!?) The names need converting
@@ -345,6 +417,9 @@ emit_event (AtkObject  *obj,
 
   dbus_connection_send(bus, sig, NULL);
   dbus_message_unref(sig);
+
+  if (g_strcmp0 (cname, "ChildrenChanged") != 0)
+    spi_object_lease_if_needed (G_OBJECT (obj));
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1018,4 +1093,17 @@ spi_atk_tidy_windows (void)
     }
 }
 
+gboolean
+spi_event_is_subtype (gchar **needle, gchar **haystack)
+{
+  while (*haystack && **haystack)
+    {
+      if (g_strcmp0 (*needle, *haystack))
+        return FALSE;
+      needle++;
+      haystack++;
+    }
+  return TRUE;
+}
+
 /*END------------------------------------------------------------------------*/
